@@ -1,6 +1,6 @@
 import json
 import logging
-from typing import Any, Dict, List, Optional, TypedDict
+from typing import Any, Iterable, Optional, TypedDict
 
 import spm_pb2
 import spm_pb2_grpc
@@ -9,7 +9,7 @@ log = logging.getLogger(__name__)
 
 
 class ScenarioSetting(TypedDict):
-    path: List[str]
+    path: list[str]
     type: str
     value: float
     id: str
@@ -18,7 +18,7 @@ class ScenarioSetting(TypedDict):
 
 class OptimizationSetting:
 
-    def __init__(self, path: List[str], setting_type: str, value: Any, raw: Any):
+    def __init__(self, path: list[str], setting_type: str, value: Any, raw: Any):
         self.path = path
         self.type = setting_type
         self.value = value
@@ -29,8 +29,8 @@ class OptimizationSetting:
                 'value': value, 'path': self.path}
 
 
-def extract_component(components: Dict[str, Any], component: Dict[str, Any]):
-    value: Dict[str, Dict] = {'params': {}, 'components': {}}
+def extract_component(components: dict[str, Any], component: dict[str, Any]):
+    value: dict[str, dict[str, Any]] = {'params': {}, 'components': {}}
     for param in component.get('parameters', []):
         value['params'][param['id']] = param.get('value')
     for subcomponent_ref in component.get('subcomponents', []):
@@ -48,15 +48,30 @@ class OptimizationConfiguration:
         if scenario.get('optimize', False):
             raise RuntimeError(f"Optimization was not enabled on scenario {scenario_id} in project {project_id}.")
 
-        # Load target scenario data
+        target_scenario = self._load_target_scenario_data_and_set_job_id(spm_stub, project_id, scenario)
+        self.settings = list(self._parse_optimization_settings(target_scenario))
+        components = self._components_by_uuid(scenario)
+        self._parse_aggregator_configuration(components)
+        self._parse_goal_function_configuration(components)
+
+    def _load_target_scenario_data_and_set_job_id(
+            self,
+            spm_stub: spm_pb2_grpc.SPMStub,
+            project_id: str,
+            scenario: dict[str, Any]
+    ) -> dict:
         self.target_scenario_id = scenario['optimizationTargetScenarioID']
         target_scenario_request = spm_pb2.Scenario(project=project_id, scenario=self.target_scenario_id)
         target_scenario_response = spm_stub.GetScenario(target_scenario_request)
         target_scenario = json.loads(target_scenario_response.scenario_document)
-        self.target_job_id = next(job_id for job_id, job in target_scenario['jobs'].items() if job.get('isMain', False))
 
-        # Validate and store optimizer configurations for parameters & inputs
-        self.settings: List[OptimizationSetting] = []
+        if not isinstance(target_scenario, dict):
+            raise TypeError(f'Received invalid target scenario data: {target_scenario}')
+
+        self.target_job_id = next(job_id for job_id, job in target_scenario['jobs'].items() if job.get('isMain', False))
+        return target_scenario
+
+    def _parse_optimization_settings(self, target_scenario: dict[str, Any]) -> Iterable[OptimizationSetting]:
         setting_paths = target_scenario.get('optimizationSettingsPaths', {})
         setting_paths.pop('__CONVERTED_FROM_MAP', None)  # Remove helper attribute
         settings_components = target_scenario.get('optimizationSettingsSimComponents', {})
@@ -66,23 +81,27 @@ class OptimizationConfiguration:
             setting_path = setting_paths.get(settings_component_uuid)
             setting_type = settings_component_uuid.split('_')[-1]  # param, value, offset, scale
             setting_value = extract_component(settings_components, settings_component)
-            component: Optional[Dict] = next((c for c in target_scenario.get('simComponents', [])
-                                              if c['uuid'] == setting_path[-2]), None)
+            component: Optional[dict[str, Any]] = next(
+                (c for c in target_scenario.get('simComponents', []) if c['uuid'] == setting_path[-2]),
+                None,
+            )
             if component is None:
                 log.warning('Invalid component for optimization: %s', setting_path[-2])
                 continue
             component_key = 'parameters' if setting_type == 'param' else 'inputVariables'
             setting_raw = next((attr for attr in component[component_key] if attr['uuid'] == setting_path[-1]), None)
-            setting_raw = self._add_setting_attrs_from_scenario(scenario, component_key, setting_path, setting_raw)
+            setting_raw = self._add_setting_attrs_from_scenario(
+                target_scenario,
+                component_key,
+                setting_path,
+                setting_raw,
+            )
             if None in (setting_path, setting_type, setting_value):
                 log.warning('Invalid optimization setting component: %s', settings_component)
                 continue
-            self.settings.append(OptimizationSetting(setting_path, setting_type, setting_value, setting_raw))
+            yield OptimizationSetting(setting_path, setting_type, setting_value, setting_raw)
 
-        # Create a dictionary of components for extracting aggregator and goal function components
-        components = {comp['uuid']: comp for comp in scenario.get('simComponents', [])}
-
-        # Configure aggregator
+    def _parse_aggregator_configuration(self, components: dict[str, dict[str, Any]]):
         self.aggregator_config = None
         aggregator_component_uuid = next((comp['uuid'] for comp in components.values() if comp['name'] == 'aggregator'),
                                          None)
@@ -92,7 +111,7 @@ class OptimizationConfiguration:
         aggregator_component = components.get(aggregator_component_uuid, {})
         self.aggregator_config = extract_component(components, aggregator_component)
 
-        # Configure goal function
+    def _parse_goal_function_configuration(self, components: dict[str, dict[str, Any]]):
         self.goal_function_inputs = []
         self.goal_function_config = None
         goal_function_component_uuid = next((comp['uuid'] for comp in components.values()
@@ -113,8 +132,16 @@ class OptimizationConfiguration:
             })
 
     @staticmethod
-    def _add_setting_attrs_from_scenario(scenario: dict, component_key: str, path: list[str],
-                                         setting_raw: Optional[dict]) -> Optional[dict]:
+    def _components_by_uuid(scenario: dict[str, Any]) -> dict[str, dict[str, Any]]:
+        return {comp['uuid']: comp for comp in scenario.get('simComponents', [])}
+
+    @staticmethod
+    def _add_setting_attrs_from_scenario(
+            scenario: dict,
+            component_key: str,
+            path: list[str],
+            setting_raw: Optional[dict]
+    ) -> Optional[dict[str, Any]]:
         attrs = ('id', 'displayName')
         component: dict = next((c for c in scenario.get('simComponents', []) if c['uuid'] == path[-2]), {})
         setting: Optional[dict] = next((s for s in component.get(component_key, []) if s['uuid'] == path[-1]), None)
